@@ -36,28 +36,28 @@ struct GceStyleClass_
 static GType          gce_style_type = 0;
 static GtkStyleClass *gce_style_parent_class = NULL;
 
-static ccd_selector_group_t *
+static ccss_selector_group_t *
 setup (GceNode const *node, 
        GceNode const *base)
 {
-	ccd_stylesheet_t const	*stylesheet;
-	ccd_selector_group_t	*group;
+	ccss_stylesheet_t const	*stylesheet;
+	ccss_selector_group_t	*group;
 	gboolean		 status;
 
 	stylesheet = gce_rc_style_get_stylesheet ();
-	group = ccd_selector_group_new ();
+	group = ccss_selector_group_new ();
 	status = FALSE;
 
 	/* query for widget theming */
-	status |= ccd_stylesheet_query_collect (stylesheet, (ccd_node_t const *) node, group, FALSE);
+	status |= ccss_stylesheet_query_collect (stylesheet, (ccss_node_t const *) node, group, FALSE);
 
 	/* query for primitive theming */
-	status |= ccd_stylesheet_query_collect (stylesheet, (ccd_node_t const *) base, group, TRUE);
+	status |= ccss_stylesheet_query_collect (stylesheet, (ccss_node_t const *) base, group, TRUE);
 
 	if (!status) {
 		g_warning ("Un-themed widget `%s', primitive `%s'.", 
-			   G_OBJECT_TYPE_NAME (G_OBJECT (node->widget)), base->impl.primitive);
-		ccd_selector_group_free (group);
+			   G_OBJECT_TYPE_NAME (G_OBJECT (gce_node_get_widget (node))), gce_node_get_primitive (base));
+		ccss_selector_group_free (group);
 		group = NULL;
 	}
 
@@ -77,14 +77,14 @@ rectangle (GtkStyle		*self,
 	   gint			 height,
 	   gboolean		 fill)
 {
-	ccd_selector_group_t	*group;
-	ccd_style_t		 style;
+	ccss_selector_group_t	*group;
+	ccss_style_t		 style;
 	cairo_t			*cr;
 
 	group = setup (node, base);
 	if (group) {
 		cr = gdk_cairo_create (window);
-		ccd_style_init (&style);
+		ccss_style_init (&style);
 
 		if (area) {
 			gdk_cairo_rectangle (cr, area);
@@ -99,15 +99,21 @@ rectangle (GtkStyle		*self,
 			height = height == -1 ? h : height;
 		}
 
-		ccd_selector_group_apply (group, &style);
+		ccss_selector_group_apply (group, &style);
+
+		/* Hackishly support "background-attachment: fixed". */
+		ccss_style_set_viewport (&style,
+			widget->allocation.x, widget->allocation.y,
+			widget->allocation.width, widget->allocation.height);
+
 		if (fill) {
-			ccd_style_draw_rectangle (&style, cr, x, y, width, height);
+			ccss_style_draw_rectangle (&style, cr, x, y, width, height);
 		} else {
-			ccd_style_draw_outline (&style, cr, x, y, width, height);
+			ccss_style_draw_outline (&style, cr, x, y, width, height);
 		}
 
 		cairo_destroy (cr);
-		ccd_selector_group_free (group);
+		ccss_selector_group_free (group);
 	}
 }
 
@@ -126,14 +132,14 @@ gap (GtkStyle		*self,
      gint		 gap_start,
      gint		 gap_width)
 {
-	ccd_selector_group_t	*group;
-	ccd_style_t		 style;
+	ccss_selector_group_t	*group;
+	ccss_style_t		 style;
 	cairo_t			*cr;
 
 	group = setup (node, base);
 	if (group) {
 		cr = gdk_cairo_create (window);
-		ccd_style_init (&style);
+		ccss_style_init (&style);
 
 		if (area) {
 			gdk_cairo_rectangle (cr, area);
@@ -148,11 +154,17 @@ gap (GtkStyle		*self,
 			height = height == -1 ? h : height;
 		}
 
-		ccd_selector_group_apply (group, &style);
-		ccd_style_draw_gap (&style, cr, x, y, width, height, gap_side, gap_start, gap_width);
+		ccss_selector_group_apply (group, &style);
+
+		/* Hackishly support "background-attachment: fixed". */
+		ccss_style_set_viewport (&style,
+			widget->allocation.x, widget->allocation.y,
+			widget->allocation.width, widget->allocation.height);
+
+		ccss_style_draw_gap (&style, cr, x, y, width, height, gap_side, gap_start, gap_width);
 
 		cairo_destroy (cr);
-		ccd_selector_group_free (group);
+		ccss_selector_group_free (group);
 	}
 }
 
@@ -168,25 +180,31 @@ line (GtkStyle		*self,
       gint		 y1, 
       gint		 y2)
 {
-	ccd_selector_group_t	*group;
-	ccd_style_t		 style;
+	ccss_selector_group_t	*group;
+	ccss_style_t		 style;
 	cairo_t			*cr;
 
 	group = setup (node, base);
 	if (group) {
 		cr = gdk_cairo_create (window);
-		ccd_style_init (&style);
+		ccss_style_init (&style);
 
 		if (area) {
 			gdk_cairo_rectangle (cr, area);
 			cairo_clip (cr);
 		}
 
-		ccd_selector_group_apply (group, &style);
-		ccd_style_draw_line (&style, cr, x1, x2, y1, y2);
+		ccss_selector_group_apply (group, &style);
+
+		/* Hackishly support "background-attachment: fixed". */
+		ccss_style_set_viewport (&style,
+			widget->allocation.x, widget->allocation.y,
+			widget->allocation.width, widget->allocation.height);
+
+		ccss_style_draw_line (&style, cr, x1, x2, y1, y2);
 
 		cairo_destroy (cr);
-		ccd_selector_group_free (group);
+		ccss_selector_group_free (group);
 	}
 }
 
@@ -204,18 +222,16 @@ draw_hline (GtkStyle		*self,
 	GceNode *node;
 	GceNode *base;
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "hline";
+	base = gce_node_cache_fetch_primitive_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			NULL, NULL, NULL, NULL, NULL,
+			"hline");
 
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget,
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			NULL, NULL, NULL, NULL, NULL);
 
 	line (self, node, base, window, area, widget, x1, x2, y, y);
 
@@ -237,18 +253,16 @@ draw_vline (GtkStyle		*self,
 	GceNode *node;
 	GceNode *base;
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "vline";
+	base = gce_node_cache_fetch_primitive_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			NULL, NULL, NULL, NULL, NULL,
+			"vline");
 
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget,
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			NULL, NULL, NULL, NULL, NULL);
 
 	line (self, node, base, window, area, widget, x, x, y1, y2);
 
@@ -273,20 +287,18 @@ draw_shadow (GtkStyle		*self,
 	GceNode *node;
 	GceNode *base;
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->shadow = gce_maps_get_shadow (shadow);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "shadow";
+	base = gce_node_cache_fetch_primitive_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, NULL, NULL, NULL,
+			"shadow");
 
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->shadow = base->shadow;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, NULL, NULL, NULL);
 
 	rectangle (self, node, base, window, area, widget, 
 		   x, y, width, height, TRUE);
@@ -311,20 +323,18 @@ draw_box (GtkStyle		*self,
 	GceNode *node;
 	GceNode *base;
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->shadow = gce_maps_get_shadow (shadow);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "box";
+	base = gce_node_cache_fetch_primitive_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, NULL, NULL, NULL,
+			"box");
 
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->shadow = base->shadow;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, NULL, NULL, NULL);
 
 	rectangle (self, node, base, window, area, widget,
 		   x, y, width, height, TRUE);
@@ -349,20 +359,18 @@ draw_flat_box (GtkStyle		*self,
 	GceNode *node;
 	GceNode *base;
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->shadow = gce_maps_get_shadow (shadow);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "flatbox";
+	base = gce_node_cache_fetch_primitive_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, NULL, NULL, NULL,
+			"flatbox");
 
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->shadow = base->shadow;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, NULL, NULL, NULL);
 
 	rectangle (self, node, base, window, area, widget, 
 		   x, y, width, height, TRUE);
@@ -387,20 +395,18 @@ draw_check (GtkStyle		*self,
 	GceNode *node;
 	GceNode *base;
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->shadow = gce_maps_get_shadow (shadow);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "check";
+	base = gce_node_cache_fetch_primitive_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, NULL, NULL, NULL,
+			"check");
 
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->shadow = base->shadow;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, NULL, NULL, NULL);
 
 	rectangle (self, node, base, window, area, widget, 
 		   x, y, width, height, TRUE);
@@ -425,20 +431,18 @@ draw_option (GtkStyle		 *self,
 	GceNode *node;
 	GceNode *base;
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->shadow = gce_maps_get_shadow (shadow);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "option";
+	base = gce_node_cache_fetch_primitive_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, NULL, NULL, NULL,
+			"option");
 
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->shadow = base->shadow;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, NULL, NULL, NULL);
 
 	rectangle (self, node, base, window, area, widget, 
 		   x, y, width, height, TRUE);
@@ -466,21 +470,18 @@ draw_shadow_gap (GtkStyle		*self,
 	GceNode *node;
 	GceNode *base;
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->gap = gce_maps_get_position (gap_side);
-	base->shadow = gce_maps_get_shadow (shadow);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "shadow";
+	base = gce_node_cache_fetch_primitive_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, gce_maps_get_position (gap_side), NULL, NULL,
+			"shadow");
 
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->shadow = base->shadow;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, gce_maps_get_position (gap_side), NULL, NULL);
 
 	gap (self, node, base, window, area, widget, 
 	     x, y, width, height, gap_side, gap_start, gap_width);
@@ -508,21 +509,18 @@ draw_box_gap (GtkStyle			*self,
 	GceNode *node;
 	GceNode *base;
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->gap = gce_maps_get_position (gap_side);
-	base->shadow = gce_maps_get_shadow (shadow);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "box";
+	base = gce_node_cache_fetch_primitive_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, gce_maps_get_position (gap_side), NULL, NULL,
+			"box");
 
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->shadow = base->shadow;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, gce_maps_get_position (gap_side), NULL, NULL);
 
 	gap (self, node, base, window, area, widget, 
 	     x, y, width, height, gap_side, gap_start, gap_width);
@@ -549,21 +547,18 @@ draw_extension (GtkStyle	*self,
 	GceNode *base;
 	gint	 gap_width;
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->gap = gce_maps_get_position (gap_side);
-	base->shadow = gce_maps_get_shadow (shadow);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "extension";
+	base = gce_node_cache_fetch_primitive_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, gce_maps_get_position (gap_side), NULL, NULL,
+			"extension");
 
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->shadow = base->shadow;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, gce_maps_get_position (gap_side), NULL, NULL);
 
 	switch (gap_side) {
 	case GTK_POS_LEFT:
@@ -600,22 +595,20 @@ draw_slider (GtkStyle		*self,
 	GceNode *node;
 	GceNode *base;
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->shadow = gce_maps_get_shadow (shadow);
-	base->orientation = gce_maps_get_orientation (orientation);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "slider";
+	base = gce_node_cache_fetch_primitive_node (widget,
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			gce_maps_get_orientation (orientation),
+			NULL, NULL, NULL,
+			"slider");
 
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->shadow = base->shadow;
-	node->orientation = base->orientation;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			gce_maps_get_orientation (orientation),
+			NULL, NULL, NULL);
 
 	rectangle (self, node, base, window, area, widget, 
 		   x, y, width, height, TRUE);
@@ -641,22 +634,20 @@ draw_handle (GtkStyle		*self,
 	GceNode *node;
 	GceNode *base;
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->shadow = gce_maps_get_shadow (shadow);
-	base->orientation = gce_maps_get_orientation (orientation);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "handle";
+	base = gce_node_cache_fetch_primitive_node (widget,
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			gce_maps_get_orientation (orientation),
+			NULL, NULL, NULL,
+			"handle");
 
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->shadow = base->shadow;
-	node->orientation = base->orientation;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			gce_maps_get_orientation (orientation),
+			NULL, NULL, NULL);
 
 	rectangle (self, node, base, window, area, widget, 
 		   x, y, width, height, TRUE);
@@ -681,20 +672,16 @@ draw_resize_grip (GtkStyle		*self,
 	GceNode *node;
 	GceNode *base;
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->edge = gce_maps_get_window_edge (edge);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "resizegrip";
+	base = gce_node_cache_fetch_primitive_node (widget,
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			NULL, NULL, NULL, gce_maps_get_window_edge (edge), NULL,
+			"resizegrip");
 
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->edge = base->edge;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			NULL, NULL, NULL, gce_maps_get_window_edge (edge), NULL);
 
 	rectangle (self, node, base, window, area, widget, 
 		   x, y, width, height, TRUE);
@@ -718,18 +705,16 @@ draw_focus (GtkStyle		*self,
 	GceNode *node;
 	GceNode *base;
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "focus";
+	base = gce_node_cache_fetch_primitive_node (widget,
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			NULL, NULL, NULL, NULL, NULL,
+			"focus");
 
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			NULL, NULL, NULL, NULL, NULL);
 
 	rectangle (self, node, base, window, area, widget, 
 		   x, y, width, height, FALSE);
@@ -764,21 +749,16 @@ draw_expander (GtkStyle		*self,
 		expander_size = DEFAULT_EXPANDER_SIZE;
 	}
 
+	base = gce_node_cache_fetch_primitive_node (widget,
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			NULL, NULL, NULL, NULL, gce_maps_get_expander_style (style),
+			"expander");
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->expander_style = gce_maps_get_expander_style (style);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "expander";
-
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->expander_style = base->expander_style;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			NULL, NULL, NULL, NULL, gce_maps_get_expander_style (style));
 
 	rectangle (self, node, base, window, area, widget, 
 		   x, y, expander_size, expander_size, TRUE);
@@ -804,20 +784,18 @@ draw_diamond (GtkStyle		*self,
 	GceNode *node;
 	GceNode *base;
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->shadow = gce_maps_get_shadow (shadow);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "diamond";
+	base = gce_node_cache_fetch_primitive_node (widget,
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, NULL, NULL, NULL,
+			"diamond");
 
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->shadow = base->shadow;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, NULL, NULL, NULL);
 
 	rectangle (self, node, base, window, area, widget, 
 		   x, y, width, height, TRUE);
@@ -826,6 +804,9 @@ draw_diamond (GtkStyle		*self,
 	gce_node_cache_release_node (base);
 }
 
+/*
+ * TODO: handle `fill'?
+ */
 static void
 draw_arrow (GtkStyle		*self,
 	    GdkWindow		*window,
@@ -844,22 +825,20 @@ draw_arrow (GtkStyle		*self,
 	GceNode *node;
 	GceNode *base;
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->shadow = gce_maps_get_shadow (shadow);
-	base->orientation = gce_maps_get_arrow (arrow);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "arrow";
+	base = gce_node_cache_fetch_primitive_node (widget,
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			gce_maps_get_arrow (arrow),
+			NULL, NULL, NULL,
+			"arrow");
 
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->shadow = base->shadow;
-	node->orientation = base->orientation;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			gce_maps_get_arrow (arrow),
+			NULL, NULL, NULL);
 
 	rectangle (self, node, base, window, area, widget, 
 		   x, y, width, height, TRUE);
@@ -877,27 +856,25 @@ draw_tab (GtkStyle		*self,
 	  GtkWidget		*widget,
 	  char const		*detail,
 	  gint			 x,
-	  gint			y,
-	  gint			width,
-	  gint			height)
+	  gint			 y,
+	  gint			 width,
+	  gint			 height)
 {
 	GceNode *node;
 	GceNode *base;
 
-	base = gce_node_cache_fetch_node ();
-	base->widget = widget;
-	base->class_name = gce_maps_get_detail (detail);
-	base->pseudo_class = gce_maps_get_state (state);
-	base->shadow = gce_maps_get_shadow (shadow);
-	base->flavor = PRIMITIVE;
-	base->impl.primitive = "tab";
+	base = gce_node_cache_fetch_primitive_node (widget,
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, NULL, NULL, NULL,
+			"tab");
 
-	node = gce_node_cache_fetch_node ();
-	node->widget = widget;
-	node->class_name = base->class_name;
-	node->pseudo_class = base->pseudo_class;
-	node->shadow = base->shadow;
-	node->flavor = UNSET;
+	node = gce_node_cache_fetch_node (widget, 
+			gce_maps_get_detail (detail),
+			gce_maps_get_state (state),
+			gce_maps_get_shadow (shadow),
+			NULL, NULL, NULL, NULL);
 
 	rectangle (self, node, base, window, area, widget, 
 		   x, y, width, height, TRUE);
